/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
// Helper method to call rest service
function callRestService(methodName, args, callbackObject, successMethodName, failureMethodName) {
	var isGet = (methodName.indexOf("get") == 0);
    var url = urlPrefix + "service/com.ibm.team.reports.common.internal.service.IReportRestService/";
    var actualMethodName = getActualMethodName(methodName);
    url += actualMethodName;
    if (isGet) { 
    	url += "?" + dojo.objectToQuery(args);
    }
    var xmlHttpRequest = createXmlHttpRequest();
    function stateChanged() {
	    if (xmlHttpRequest.readyState == 4 || xmlHttpRequest.readyState=="complete") {
	    	if (xmlHttpRequest.status != 200) {
	    		var message = "An error occurred while processing the request.";
	    		callbackObject[failureMethodName].call(callbackObject, {message: message}, message);
	    		return;
	    	}
	    	var evaldObj = parseResponse(xmlHttpRequest, methodName);
	    	callbackObject[successMethodName].call(callbackObject, evaldObj);
	    } 
    }
    xmlHttpRequest.onreadystatechange = stateChanged;
    if (methodName.indexOf("post") == 0) {
    	xmlHttpRequest.open("POST", url, true);
    } else {
	    xmlHttpRequest.open("GET", url, true);
	}
	xmlHttpRequest.setRequestHeader("Accept", "text/xml");
    xmlHttpRequest.setRequestHeader("X-com-ibm-team-userid", ownerName);
   	xmlHttpRequest.setRequestHeader("Content-Type", "application/x-www-form-urlencoded; charset=utf-8");
   	if (isGet) {
    	xmlHttpRequest.send(null);
    } else {
    	xmlHttpRequest.send(dojo.objectToQuery(args));
    }
}

function parseResponse(xmlHttpRequest, methodName) {
	if (methodName == "getParameterValues") {
		// Response is a tree structure
		var values = xmlHttpRequest.responseXML.getElementsByTagName("value");
		if (!values) return null;
		return parseReportParameterDTO(values[0]);
	}
	if (methodName == "getParameterGroupValues") {
		// Response is a tree structure
		var values = xmlHttpRequest.responseXML.getElementsByTagName("value");
		if (!values) return null;
		return parseReportParameterGroupDTO(values[0]);
	}
	if (methodName == "getRenderedReport") {
		var value = xmlHttpRequest.responseXML.getElementsByTagName("value")[0];
		var result = "";
		for (var i = 0; i < value.childNodes.length; i++) {
			result += value.childNodes[i].nodeValue;
		}
		return result;
	}
	if (methodName == "postRenderReport") {
		var value = xmlHttpRequest.responseXML.getElementsByTagName("value")[0];
		var result = "";
		for (var i = 0; i < value.childNodes.length; i++) {
			result += value.childNodes[i].nodeValue;
		}
		return result;
	}
	
	return null;
}

function parseReportParameterGroupDTO(element) {
	var parametersElement = element.getElementsByTagName("parameters")[0];
	var parameters = [parseParameter(parametersElement)];
	var result = {
		parameters: parameters
	};
	return result;
}

function parseReportParameterDTO(element) {
	var result = {
		name: findElementValue(element, "name"),
		parameterName: findElementValue(element, "parameterName"),
		flags: findElementValue(element, "flags"),
		values: findElementValues(element, "values"),
		labels: findElementValues(element, "labels")
	};
	return result;
}

function parseParameter(parametersElement) {
	var parameterName = findElementValue(parametersElement, "parameterName");
	var flags = findElementValue(parametersElement, "flags");
	var valuesSoFar = findElementValues(parametersElement, "valuesSoFar");
	var values = findElementValues(parametersElement, "values");
	var labels = findElementValues(parametersElement, "labels");
	var children = findParameterChildren(parametersElement);
	var result = {
		parameterName: parameterName,
		flags: flags,
		valuesSoFar: valuesSoFar,
		values: values,
		labels: labels,
		children: children
	}; 
	return result;
}

function findParameterChildren(parametersElement) {
	var result = [];
	var index = 0;
	for (var i = 0; i < parametersElement.childNodes.length; i++) {
		if (parametersElement.childNodes[i].nodeName == "children") {
			result[index++] = parseParameter(parametersElement.childNodes[i]);
		}
	}
	if (result.length > 0) return result;
	return undefined;
}

function findElementValue(parametersElement, childElementName) {
	for (var i = 0; i < parametersElement.childNodes.length; i++) {
		if (parametersElement.childNodes[i].nodeName == childElementName) {
			var childNode = parametersElement.childNodes[i].childNodes[0];
			if (childNode != undefined && childNode != null) {
				return childNode.nodeValue;
			}
		}
	}
	return undefined;
}

function findElementValues(parametersElement, childElementName) {
	var result = [];
	var index = 0;
	for (var i = 0; i < parametersElement.childNodes.length; i++) {
		if (parametersElement.childNodes[i].nodeName == childElementName) {
			var childNode = parametersElement.childNodes[i].childNodes[0];
			if (childNode != undefined && childNode != null) {
				result[index++] = childNode.nodeValue;
			}
		}
	}
	if (result.length > 0) return result;
	return undefined;
}

function updateReport(useInitialValues, warnIfMissingParameters) {
	if (useInitialValues == null || useInitialValues == undefined) useInitialValues = false;
	if (warnIfMissingParameters == null || warnIfMissingParameters == undefined) warnIfMissingParameters = true;
    if (!useInitialValues && !hasAllRequiredParameters()) {
    	if (warnIfMissingParameters) {
    		// Tell the user and don't do anything.
	        alert("Please enter values for all required parameters");
	    } else {
	    	// Display an empty page.
	    	var div = document.getElementById("renderedReportDiv");
	    	div.innerHTML = "";
	    }
        return;
    }
    
    var obj = {
        mySuccessFunction: mySuccessFunction,
        myErrorFunction: myErrorFunction
    };
    
    
    // Install a loading indicator before sending the request
    var div = document.getElementById("report");
    div.innerHTML = "<div class=\"loading\">Loading...</div>";
    
    var oldParameterValues = null;
    if (useInitialValues && useInitialValues == true) {
    	oldParameterValues = initialParameterValues;
    } else {
    	oldParameterValues = allParameterValues;
    }
	var newParameters = prepareParameters(oldParameterValues);    
    
    var args = {
        reportUUID: reportUUID,
        embeddable: true,
        parameterName: newParameters.names,
        parameterValue: newParameters.values,
        webui: false,
        showArchived: showArchived
    };
    callRestService("postRenderReport", 
        args, obj, 
        "mySuccessFunction", "myErrorFunction");
}

function mySuccessFunction(args) {
    // Insert the HTML into the report div
	if (navigator.appVersion.indexOf("MSIE")!=-1) {
		// Insert innerHTML directly. That, coupled with defer tags
		// on the scripts, causes new Javascript to be executed properly.
        var div = document.getElementById("report");
        div.innerHTML = args;
	} else {
		// Create a new node and appendChild so javascript gets executed.
		var newNode = document.createElement('div');
		newNode.innerHTML = args;
        var div = document.getElementById("report");
		while (div.firstChild) {
		    div.removeChild(div.firstChild);
		}
		div.appendChild(newNode);
	} 
}

function myErrorFunction(errorType, errorMessage) {
	alert("Error: " + errorType.message);
}

function createXmlHttpRequest() {
   var result = null;
    if (window.XMLHttpRequest) {
        result = new XMLHttpRequest();
    } else if (window.ActiveXObject) {
        result = new ActiveXObject("Microsoft.XMLHTTP");
    }
    return result;
}

function postSave() {
    window.location = "http://com.ibm.team.reports/new_query/";	
}

function postSetAsDefault(defaultQuery) {
	// Do nothing
}

function getActualMethodName(methodName) {
	var regex = /^(get|post)/;
	var methodType = methodName.match(regex)[0].toUpperCase();
	var actualMethodName = methodName.substring(methodType.length, methodType.length + 1).toLowerCase() +
		methodName.substring(methodType.length + 1);
	return actualMethodName;
}