/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.TeamAreaEditor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");
dojo.require("dojo.cookie");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.ui.internal.Dialog");

dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ProcessRoleDialog");
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.TeamParentSelector");
dojo.require("com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog");

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var MANAGER = "com.ibm.team.process.manageProjectAreas";
var ARCHIVED_MANAGER = "com.ibm.team.process.manageArchivedProjectAreas";
var PROJECT_AREA_EDITOR = "com.ibm.team.process.editProjectArea";
var TEAM_AREA_EDITOR = "com.ibm.team.process.editTeamArea";
var USER_EDITOR = "com.ibm.team.repository.editUser";
var ARCHIVED_USER_EDITOR = "com.ibm.team.repository.editArchivedUser";
var MSG_COOKIE_NAME = "com.ibm.team.process.editTeamArea#saveMsg";
var KEY_RETURN = 13;
var KEY_ENTER = 14;
var DEFAULT_DISPLAY_MAX = 5;
var SHOW_TEXT_LENGTH = 1024;

dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.TeamAreaEditor", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/TeamAreaEditor.html"),
	
	constructor: function(args){
		this.isNew = args.itemId === "new";
		if (!this.isNew) {
			this.itemId = args.itemId;
		}
		this.parentItemId = args.parentItemId;
		this.parentName = args.parentName;
		this.parentType = args.parentType;
		this._teamAreaIconSrc = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/teamAreaIcon.png");
	},
	
	postCreate: function () {
		this.showingAllMembers = false;
		this.showingAllAdmins = false;
		this.connectHandles = [];
		this.inputFields = [
			this.name,
			this.summary,
			this.description
		];
		this.connect(this.name, "onkeypress", function (e) {
			if (e.keyCode === KEY_RETURN || e.keyCode === KEY_ENTER) {
				dojo.stopEvent(e);
				this.togglePrimaryField("Preview");
			}
		});
		this.summary.viewElement = this.summaryView;
		this.summary.viewAllElement = this.summaryViewAll;
		this.summary.showAllLink = this.showAllSummaryLink;
		this.summary.showLessLink = this.showLessSummaryLink;
		this.summary.showingAll = false;
		this.description.viewElement = this.descriptionView;
		this.description.viewAllElement = this.descriptionViewAll;
		this.description.showAllLink = this.showAllDescriptionLink;
		this.description.showLessLink = this.showLessDescriptionLink;
		this.description.showingAll = false;
		this.editViewFields = [
			this.summary,
			this.description
		];
		this.connect(this.editViewTable,"onclick", function () {
			this.toggleViewEditMode("Edit");
		});
		for (var i=0; i < this.inputFields.length; i++) {
			this.connect(this.inputFields[i],"onkeyup", "handleChange");
			this.connect(this.inputFields[i],"onchange", "handleChange");
		}
		this.connect(this.memberAddButton,"onclick", function (e) {
			dojo.stopEvent(e);
			if (this.allMembers == null) {
				this.allMembers = new Array();
			}
			this.addUserDialog(this.allMembers, "addNewMembers");
		});
		this.connect(this.adminAddButton,"onclick", function (e) {
			dojo.stopEvent(e);
			if (this.allAdmins == null) {
				this.allAdmins = new Array();
			}
			this.addUserDialog(this.allAdmins, "addNewAdmins");
		});
		this.connect(this.saveButton,"onclick", "handleSave");
		this.connect(this.name,"onchange", function () {
			if (this.hierarchyNameDisplay != null) {
				dojox.data.dom.removeChildren(this.hierarchyNameDisplay);
				if (this.name.value == "") {
					this.hierarchyNameDisplay.appendChild(document.createTextNode("(New Team Area)"));
				}
				else {
					this.hierarchyNameDisplay.appendChild(document.createTextNode(this.name.value));
				}
			}
		});
		this.connect(this.toggleDescLink,"onclick", function (e) {
			dojo.stopEvent(e);
			this.toggleViewEditMode(e.target.innerHTML);
		});
		
		this.connect(this.previewName,"onmouseover", function (e) {
			e.target.style.backgroundColor = "#FFFFCC";
		});
		this.connect(this.previewName,"onmouseout", function (e) {
			e.target.style.backgroundColor = "";
		});
		this.connect(this.previewName,"onclick", function (e) {
			this.togglePrimaryField("Edit");
		});
		this.connect(this.name, "onblur", function (e) {
			this.togglePrimaryField("Preview");
		});
		
		if (!this.isNew) {
			this.createChildButton.style.display = "";
			this.connect(this.createChildButton,"onclick", function (e) {
				dojo.stopEvent(e);
				if (!this._handleNavigate()) {
					document.location.href = "#action=" + TEAM_AREA_EDITOR + "&itemId=new&parentItemId=" + this.itemId + "&parentType=TEAM";
				}
			});
		}
		this.connect(this.showAllSummaryLink, "onclick", function(e) {
			dojo.stopEvent(e);
			this.summary.showingAll = true;
			this.summary.viewAllElement.style.display = "inline";
			this.summary.showAllLink.style.display = "none";
			this.summary.showLessLink.style.display = "block";
		});
		this.connect(this.showLessSummaryLink, "onclick", function(e) {
			dojo.stopEvent(e);
			this.summary.showingAll = false;
			this.summary.viewAllElement.style.display = "none";
			this.summary.showAllLink.style.display = "block";
			this.summary.showLessLink.style.display = "none";
		});
		this.connect(this.showAllDescriptionLink, "onclick", function(e) {
			dojo.stopEvent(e);
			this.description.showingAll = true;
			this.description.viewAllElement.style.display = "inline";
			this.description.showAllLink.style.display = "none";
			this.description.showLessLink.style.display = "block";
		});
		this.connect(this.showLessDescriptionLink, "onclick", function(e) {
			dojo.stopEvent(e);
			this.description.showingAll = false;
			this.description.viewAllElement.style.display = "none";
			this.description.showAllLink.style.display = "block";
			this.description.showLessLink.style.display = "none";
		});
		this.connect(this.showAllMembersLink, "onclick", function(e) {
			dojo.stopEvent(e);
			this.showingAllMembers = true;
			this.showAllMembersLink.style.display = "none";
			this.showFewerMembersLink.style.display = "block";
			for (var i=0; i < this.memberArea.rows.length; i++) {
				this.memberArea.rows[i].style.display = "";
			}
		});
		this.connect(this.showFewerMembersLink, "onclick", function(e) {
			dojo.stopEvent(e);
			this.showingAllMembers = false;
			this.showAllMembersLink.style.display = "block";
			this.showFewerMembersLink.style.display = "none";
			for (var i=DEFAULT_DISPLAY_MAX; i < this.memberArea.rows.length; i++) {
				this.memberArea.rows[i].style.display = "none";
			}
		});
		this.connect(this.showAllAdminsLink, "onclick", function(e) {
			dojo.stopEvent(e);
			this.showingAllAdmins = true;
			this.showAllAdminsLink.style.display = "none";
			this.showFewerAdminsLink.style.display = "block";
			for (var i=0; i < this.adminArea.rows.length; i++) {
				this.adminArea.rows[i].style.display = "";
			}
		});
		this.connect(this.showFewerAdminsLink, "onclick", function(e) {
			dojo.stopEvent(e);
			this.showingAllAdmins = false;
			this.showAllAdminsLink.style.display = "block";
			this.showFewerAdminsLink.style.display = "none";
			for (var i=DEFAULT_DISPLAY_MAX; i < this.adminArea.rows.length; i++) {
				this.adminArea.rows[i].style.display = "none";
			}
		});
		this.refresh();
	},
	
	addUserDialog: function (filterUsers, callbackFunction) {
		var userFilter = new Array();
		for (var i=0; i < filterUsers.length; i++) {
			userFilter.push({'itemId': filterUsers[i].itemId});
		}
		var userPicker = new com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog({'disableUsers': userFilter});
		this.connect(userPicker,"onAdd",callbackFunction);
	},
	
	refresh: function () {
		if (!this.navigateAwayHandle) {
			this.navigateAwayHandle = PlatformUI.getWorkbench().registerBodyClickHandler(dojo.hitch(this, this._handleNavigate), /*priority*/ 10000);
			this.onBeforeUnloadHandle = PlatformUI.getWorkbench().registerOnBeforeUnloadHandler(dojo.hitch(this, this.navigateAwayEvent), /*priority*/ 10000);
		}
		this.clearState();
		this.selectedMembers = new Object();
		this.selectedAdmins = new Object();
		this.pendingMemberChanges = new Object();
		this.pendingAdminChanges = new Object();
		
		dojox.data.dom.removeChildren(this.processHierarchy);
		this.processHierarchy.appendChild(document.createTextNode("Loading..."));
		dojox.data.dom.removeChildren(this.developmentLineDiv);
		this.developmentLineDiv.appendChild(document.createTextNode("Loading..."));
		
		if (this.isNew) {
			this.toggleViewEditMode("Edit");
			this.name.value = "Team Name";
			this.togglePrimaryField("Edit");
			this.populateMembers(new Array());
			this.populateAdmins(new Array());
			this.populateDevelopmentLineSelector();
			
			if (this.parentType != null && this.parentItemId != null) {
				var srh = new ServiceResponseHandler(this, "populateProcessHierarchy", "ignoreError");
				if (this.parentType == "TEAM") {
					ProcessClient.getHierarchyForTeamArea({uuid: this.parentItemId}, srh);
				}
				else if (this.parentType == "PROJECT") {
					ProcessClient.getProjectHierarchy({uuid: this.parentItemId}, srh);
				}
			}
			setTimeout(dojo.hitch(this.name,"select"), 250);
		}
		else {
			dojox.data.dom.removeChildren(this.memberArea);
			dojox.data.dom.removeChildren(this.adminArea);
			var srh = new ServiceResponseHandler(this, "populateTeamAreaData", "handleTeamAreaFetchError");
			ProcessClient.getTeamAreaByUUID({uuid: this.itemId}, srh);
		}
		this.checkForSuccessMsg();
	},
	
	_handleNavigate: function(urlObj) {
		if (this.changesPending()) {
			if (confirm("Are you sure you want to navigate away from this page?\n\nYou have unsaved changes in the team area editor.  If you continue, you will lose your changes.\n\nPress OK to continue, or Cancel to stay on the current page.")) {
				PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
				this.navigateAwayHandle = null;
				PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);
				this.onBeforeUnloadHandle = null;
				return false;
			}
			else {
				return true;
			}
		}
		return false;
	},
	
	navigateAwayEvent: function() {
		return this.changesPending() ? "You have unsaved changes in the team area editor.  If you continue, you will lose your changes." : false;
	},
	
	checkForSuccessMsg: function () {
		var msgCookie = dojo.cookie(MSG_COOKIE_NAME);
		if (msgCookie) {
			this.displaySuccessMessage(msgCookie, true);
			var now = new Date();
			now.setTime(now.getTime() - 1);
			dojo.cookie(MSG_COOKIE_NAME,"",{expires: now});
		}
	},
	
	clearState: function () {
		this.setDirty(false);
		this.selectedMembers = new Object();
		this.selectedAdmins = new Object();
		this.pendingMemberChanges = new Object();
		this.pendingAdminChanges = new Object();
		this.allMembers = new Array();
		this.allAdmins = new Array();
		this.parentProcessArea = null;
		this.name.value = "";
		this.summary.value = "";
		this.description.value = "";
		dojox.data.dom.removeChildren(this.processHierarchy);
		dojox.data.dom.removeChildren(this.developmentLineDiv);
		dojox.data.dom.removeChildren(this.memberArea);
		dojox.data.dom.removeChildren(this.adminArea);
		dojox.data.dom.removeChildren(this.messageBox);
		dojox.data.dom.removeChildren(this.longMessageBox);
		this.messageBox.className = "";
		this.longMessageBox.className = "";
		for (var i=0; i < this.inputFields.length; i++) {
			this.inputFields[i].changePending = false;
		}
	},
	
	handleTeamAreaFetchError: function (err) {
		var msg = document.createElement("div");
		msg.appendChild(document.createTextNode("Error fetching team area data.  Please check your server log for more details."));
		if (err.message != null && err.message != "") {
			msg.appendChild(document.createElement("br"));
			msg.appendChild(document.createElement("br"));
			msg.appendChild(document.createTextNode(err.message));
		}
		this.displayLongErrorMessage(msg);
	},
	
	togglePrimaryField: function (mode) {
		if (mode === "Preview") {
			this.name.style.display = "none";
			dojox.data.dom.removeChildren(this.previewName);
			this.previewName.appendChild(document.createTextNode(this.name.value || "Team Area Name"));
			this.previewName.style.display = "";
		}
		else if (mode === "Edit") {
			this.previewName.style.display = "none";
			this.name.style.display = "";
			this.name.focus();
			this.name.select();
			this.displayMessage("","");
		}
	},
	
	toggleViewEditMode: function (mode) {
		if (mode === "Preview") {
			dojo.forEach(this.editViewFields, function (field) {
				field.style.display = "none";
				dojox.data.dom.removeChildren(field.viewElement);
				var previewString = field.value.substring(0,SHOW_TEXT_LENGTH);
				var hiddenString = field.value.substring(SHOW_TEXT_LENGTH);
				if (field.value.length > SHOW_TEXT_LENGTH) {
					var spaceIndex = hiddenString.indexOf(" ");
					var newlineIndex = hiddenString.indexOf("\n");
					// if there is no good point to break the string in hiddenString, just show the whole thing
					if (spaceIndex === -1 && newlineIndex === -1) {
						previewString += hiddenString;
						hiddenString = "";
					}
					else if (newlineIndex === -1 || (spaceIndex !== -1 && spaceIndex < newlineIndex)) {
						previewString += hiddenString.substring(0, spaceIndex);
						hiddenString = hiddenString.substring(spaceIndex);
					}
					else if (spaceIndex === -1 || (newlineIndex !== -1 && newlineIndex < spaceIndex)) {
						previewString += hiddenString.substring(0, newlineIndex);
						hiddenString = hiddenString.substring(newlineIndex);
					}
				}
				var strings = previewString.split(/\n/);
				for (var i=0; i < strings.length; i++) {
					if (i > 0) {
						field.viewElement.appendChild(document.createElement("br"));
					}
					if (strings[i].length > 0) {
						field.viewElement.appendChild(document.createTextNode(strings[i]));
					}
				}
				field.viewElement.style.display = "";
				dojox.data.dom.removeChildren(field.viewAllElement);
				if (hiddenString != null && hiddenString !== "") {
					strings = hiddenString.split(/\n/);
					for (var i=0; i < strings.length; i++) {
						if (i > 0) {
							field.viewAllElement.appendChild(document.createElement("br"));
						}
						if (strings[i].length > 0) {
							field.viewAllElement.appendChild(document.createTextNode(strings[i]));
						}
					}
					if (field.showingAll) {
						field.viewAllElement.style.display = "inline";
						field.showLessLink.style.display = "block";
						field.showAllLink.style.display = "none";
					}
					else {
						field.viewAllElement.style.display = "none";
						field.showLessLink.style.display = "none";
						field.showAllLink.style.display = "block";
					}
				}
			});
			dojox.data.dom.removeChildren(this.toggleDescLink);
			this.toggleDescLink.appendChild(document.createTextNode("Edit"));
		}
		else if (mode === "Edit") {
			dojox.data.dom.removeChildren(this.toggleDescLink);
			this.toggleDescLink.appendChild(document.createTextNode("Preview"));
			dojo.forEach(this.editViewFields, function (field) {
				field.viewElement.style.display = "none";
				field.viewAllElement.style.display = "none";
				field.style.display = "";
				field.showAllLink.style.display = "none";
				field.showLessLink.style.display = "none";
			});
		}
	},
	
	populateTeamAreaData: function (data) {
		this.name.value = data.name;
		this.name.savedValue = data.name;
		this.togglePrimaryField("Preview");
		this.summary.value = data.summary;
		this.summary.savedValue = data.summary;
		this.description.value = data.description;
		this.description.savedValue = data.description;
		if (data.description) {
			this.toggleViewEditMode("Preview");
		}
		else {
			this.toggleViewEditMode("Edit");
		}
		this.allProcessRoles = data.processDTO.roles;
		this.populateProcessHierarchy(data.processHierarchy);
		this.allMembers = data.members;
		this.allAdmins = data.admins;
		this.populateMembers(this.allMembers);
		this.populateAdmins(this.allAdmins);
		this.populateDevelopmentLine(data.developmentLine);
	},
	
	populateMembers: function (members) {
		this.populateUsers(members, this.memberHeader, this.memberArea, !this.isNew, this.showingAllMembers, "memberRemoveHandler", "processRoleHandler");
		if (members != null && members.length > DEFAULT_DISPLAY_MAX) {
			if (this.showingAllMembers) {
				this.showFewerMembersLink.style.display = "block";
				this.showAllMembersLink.style.display = "none";
			}
			else {
				this.showFewerMembersLink.style.display = "none";
				this.showAllMembersLink.style.display = "block";
			}
		}
		else {
			this.showFewerMembersLink.style.display = "none";
			this.showAllMembersLink.style.display = "none";
		}
	},
	
	populateAdmins: function (admins) {
		this.populateUsers(admins, this.adminHeader, this.adminArea, false, this.showingAllAdmins, "adminRemoveHandler", null);
		if (admins != null && admins.length > DEFAULT_DISPLAY_MAX) {
			if (this.showingAllAdmins) {
				this.showFewerAdminsLink.style.display = "block";
				this.showAllAdminsLink.style.display = "none";
			}
			else {
				this.showFewerAdminsLink.style.display = "none";
				this.showAllAdminsLink.style.display = "block";
			}
		}
		else {
			this.showFewerAdminsLink.style.display = "none";
			this.showAllAdminsLink.style.display = "none";
		}
	},
	
	populateUsers: function (users, header, userArea, showRoles, showAll, removeHandler, processRoleHandler) {
		dojox.data.dom.removeChildren(header);
		var tr = document.createElement("tr");
		tr.className = "headerLineRow";
		var th = document.createElement("th");
		th.className = "memberName";
		if (dojo.isIE) {
			th.appendChild(document.createTextNode(" "));
		}
		tr.appendChild(th);
		if (showRoles) {
			th = document.createElement("th");
			th.className = "memberRoles";
			if (dojo.isIE) {
				th.appendChild(document.createTextNode(" "));
			}
			tr.appendChild(th);
		}
		else {
			th = document.createElement("th");
			th.className = "memberMail";
			if (dojo.isIE) {
				th.appendChild(document.createTextNode(" "));
			}
			tr.appendChild(th);
		}
		th = document.createElement("th");
		th.className = "iconNode";
		if (dojo.isIE) {
			th.appendChild(document.createTextNode(" "));
		}
		tr.appendChild(th);
		header.appendChild(tr);
		tr = document.createElement("tr");
		th = document.createElement("th");
		th.appendChild(document.createTextNode("Name"));
		th.className = "memberName";
		tr.appendChild(th);
		if (showRoles) {
			th = document.createElement("th");
			th.appendChild(document.createTextNode("Process Roles"));
			th.className = "memberRoles";
			tr.appendChild(th);
		}
		else {
			th = document.createElement("th");
			th.appendChild(document.createTextNode("E-mail"));
			th.className = "memberMail";
			tr.appendChild(th);
		}
		th = document.createElement("th");
		th.className = "iconNode";
		tr.appendChild(th);
		
		header.appendChild(tr);
		dojox.data.dom.removeChildren(userArea);
		if (users != null && users.length > 0) {
			for (var i=0; i < users.length; i++) {
				var tr = document.createElement("tr");
				tr.userObj = users[i];
				var td = document.createElement("td");
				td.className = "memberName";
				var link = document.createElement("a");
				if (users[i].archived) {
					link.href = "#action=" + ARCHIVED_USER_EDITOR + "&itemId=" + users[i].itemId;
				}
				else {
					link.href = "#action=" + USER_EDITOR + "&itemId=" + users[i].itemId;
				}
				link.appendChild(document.createTextNode(users[i].name));
				td.appendChild(link);
				tr.appendChild(td);
				if (showRoles) {
					td = document.createElement("td");
					td.className = "memberRoles";
					var roleStr = "";
					if (users[i].processRoles != null) {
						users[i].savedProcessRoles = users[i].processRoles;
						var roleIds = new Array();
						for (var j=0; j < users[i].processRoles.length; j++) {
							// hide the default role from the display
							if (users[i].processRoles[j].id != "default") {
								roleIds.push(users[i].processRoles[j].id);
							}
						}
						roleStr = roleIds.join(", ");
					}
					if (dojo.isIE && roleStr === "") {
						roleStr = " ";
					}
					td.appendChild(document.createTextNode(roleStr));
					tr.roleDisplay = td;
				}
				else {
					td = document.createElement("td");
					td.className = "memberMail";
					var link = document.createElement("a");
					link.href = "mailto:" + users[i].emailAddress;
					link.appendChild(document.createTextNode(users[i].emailAddress));
					td.appendChild(link);
					tr.mailDisplay = td;
				}
				tr.appendChild(td);
				td = document.createElement("td");
				td.className = "iconNode";
				if (showRoles) {
					var processRolesIcon = document.createElement("img");
					processRolesIcon.src = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/process-roles.gif");
					processRolesIcon.nohoverSrc = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/process-roles.gif");
					processRolesIcon.hoverSrc = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/process-roles-hov.gif");
					this.connect(processRolesIcon, "onclick", processRoleHandler);
					this.connectHandles.push(dojo.connect(processRolesIcon, "onmouseover", processRolesIcon, function (e) {
						this.src = this.hoverSrc;
					}));
					this.connectHandles.push(dojo.connect(processRolesIcon, "onmouseout", processRolesIcon, function (e) {
						this.src = this.nohoverSrc;
					}));
					processRolesIcon.title = "Process Roles";
					td.appendChild(processRolesIcon);
				}
				var removeIcon = document.createElement("img");
				removeIcon.src = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/remove.gif");
				removeIcon.nohoverSrc = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/remove.gif");
				removeIcon.hoverSrc = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/remove-hov.gif");
				removeIcon.title = "Remove";
				this.connect(removeIcon,"onclick",removeHandler);
				this.connectHandles.push(dojo.connect(removeIcon, "onmouseover", removeIcon, function (e) {
					this.src = this.hoverSrc;
				}));
				this.connectHandles.push(dojo.connect(removeIcon, "onmouseout", removeIcon, function (e) {
					this.src = this.nohoverSrc;
				}));
				td.appendChild(removeIcon);
				tr.appendChild(td);
				this.connectHandles.push(dojo.connect(tr, "onmouseover", tr, function (e) {
					dojo.toggleClass(this, "highlighted", true);
				}));
				this.connectHandles.push(dojo.connect(tr, "onmouseout", function (e) {
					dojo.toggleClass(this, "highlighted", false);
				}));
				if (!showAll && i >= DEFAULT_DISPLAY_MAX) {
					tr.style.display = "none";
				}
				userArea.appendChild(tr);
			}
		}
	},
	
	memberRemoveHandler: function (e) {
		if (this.pendingMemberChanges["remove"] == null) {
			this.pendingMemberChanges["remove"] = new Array();
		}
		var rowNode = e.target;
		while (rowNode.tagName !== "TR") {
			rowNode = rowNode.parentNode;
		}
		var selectedUser = rowNode.userObj;
		var itemId = selectedUser.itemId;
		
		this.pendingMemberChanges.remove.push(itemId);
		for (var j=0; j < this.allMembers.length; j++) {
			if (itemId === this.allMembers[j].itemId) {
				this.allMembers.splice(j,1);
				break;
			}
		}
		if (this.pendingMemberChanges.add != null) {
			for (var j=0; j < this.pendingMemberChanges.remove.length; j++) {
				if (this.pendingMemberChanges.add[j] == itemId) {
					this.pendingMemberChanges.add.splice(j,1);
					break;
				}
			}
		}
		this.setDirty(true);
		this.populateMembers(this.allMembers);
	},
	
	processRoleHandler: function (e) {
		var rowNode = e.target;
		while (rowNode.tagName !== "TR") {
			rowNode = rowNode.parentNode;
		}
		var selectedUser = rowNode.userObj;
		this.processRoleRow = rowNode;
		
		var workbenchNode = dojo.byId("workbench");
		var roleObj = new Object();
		roleObj.allRoles = this.allProcessRoles;
		roleObj.member = selectedUser;
		var dialogWidget = dijit.byId("com.ibm.team.process.web.ui.internal.admin.projects.ProcessRoleDialog");
		if (dialogWidget == null) {
			var processRoleDialog = new com.ibm.team.process.web.ui.internal.admin.projects.ProcessRoleDialog(roleObj, workbenchNode);
			dialogWidget = new net.jazz.ajax.ui.internal.Dialog({id:"com.ibm.team.process.web.ui.internal.admin.projects.ProcessRoleDialog",
				primaryTitle: "Edit Process Roles",
				contentNode: processRoleDialog.domNode,
				width: dojo.isIE ? "48em" : "46em",
				execute: function(selectedRoles) {
					var roleIds = new Array();
					for (var i=0; i < selectedRoles.length; i++) {
						// hide the default role from the display
						if (selectedRoles[i].id != "default") {
							roleIds.push(selectedRoles[i].id);
						}
					}
					var roleString = roleIds.join(", ");
					var selectedRow = this.targetObj.processRoleRow;
					dojox.data.dom.removeChildren(selectedRow.roleDisplay);
					selectedRow.roleDisplay.appendChild(document.createTextNode(roleString));
					selectedRow.userObj.processRoles = selectedRoles;
					var roleChanged = false;
					if (selectedRoles.length == selectedRow.userObj.savedProcessRoles.length) {
						for (var j=0; j < selectedRoles.length; j++) {
							if (selectedRoles[j].id != selectedRow.userObj.savedProcessRoles[j].id) {
								roleChanged = true;
								break;
							}
						}
					}
					else {
						roleChanged = true;
					}
					if (roleChanged) {
						var roleArray = new Array();
						for (var j=0; j < selectedRow.userObj.processRoles.length; j++) {
							roleArray.push(selectedRow.userObj.processRoles[j].id);
						}
						if (this.targetObj.pendingMemberChanges["roles"] == null) {
							this.targetObj.pendingMemberChanges["roles"] = new Object();
						}
						this.targetObj.pendingMemberChanges["roles"][selectedRow.userObj.itemId] = roleArray;
					}
					else {
						if (this.targetObj.pendingMemberChanges["roles"] != null) {
							delete this.targetObj.pendingMemberChanges["roles"][selectedRow.userObj.itemId];
							var roleChanges = false;
							for (var j in this.targetObj.pendingMemberChanges["roles"]) {
								roleChanges = true;
								break;
							}
							if (!roleChanges) {
								delete this.targetObj.pendingMemberChanges["roles"];
							}
						}
					}
					if (this.targetObj.changesPending()) {
						this.targetObj.saveButton.disabled = false;
					}
					else {
						this.targetObj.saveButton.disabled = true;
					}
				}
			});
			dialogWidget.targetObj = this;
			dialogWidget.processRoleDialogObj = processRoleDialog;
			this.connectHandles.push(dojo.connect(processRoleDialog.cancelButton, "onclick", dialogWidget, function (e) {
				dojo.stopEvent(e);
				this.close();
			}));
			this.connectHandles.push(dojo.connect(processRoleDialog.finishButton, "onclick", dialogWidget, function (e) {
				this.execute(this.processRoleDialogObj.getChosenRoles());
				this.close();
			}));
		}
		else {
			dialogWidget.processRoleDialogObj.repopulate(roleObj);
			dialogWidget.targetObj = this;
		}
	},
	
	adminRemoveHandler: function (e) {
		if (this.pendingAdminChanges["remove"] == null) {
			this.pendingAdminChanges["remove"] = new Array();
		}
		var rowNode = e.target;
		while (rowNode.tagName !== "TR") {
			rowNode = rowNode.parentNode;
		}
		var selectedUser = rowNode.userObj;
		var itemId = selectedUser.itemId;
		
		this.pendingAdminChanges.remove.push(itemId);
		for (var j=0; j < this.allAdmins.length; j++) {
			if (itemId === this.allAdmins[j].itemId) {
				this.allAdmins.splice(j,1);
				break;
			}
		}
		if (this.pendingAdminChanges.add != null) {
			for (var j=0; j < this.pendingAdminChanges.remove.length; j++) {
				if (this.pendingAdminChanges.add[j] == itemId) {
					this.pendingAdminChanges.add.splice(j,1);
					break;
				}
			}
		}
		this.setDirty(true);
		this.populateAdmins(this.allAdmins);
	},
	
	addNewMembers: function (newMembers) {
		if (newMembers != null) {
			if (newMembers.length > 0) {
				this.setDirty(true);
				if (this.pendingMemberChanges["add"] == null) {
					this.pendingMemberChanges["add"] = new Array();
				}
			}
			for (var i=0; i < newMembers.length; i++) {
				var member = newMembers[i];
				member.itemId = member.itemId;
				member.processRoles = [{id: "default"}];
				member.savedProcessRoles = [{id: "default"}];
				this.allMembers.push(member);
				this.pendingMemberChanges.add.push(member.itemId);
				if (this.pendingMemberChanges.remove != null) {
					for (var j=0; j < this.pendingMemberChanges.remove.length; j++) {
						if (this.pendingMemberChanges.remove[j] == member.itemId) {
							this.pendingMemberChanges.remove.splice(j,1);
							break;
						}
					}
				}
			}
		}
		this.populateMembers(this.allMembers);
	},
	
	addNewAdmins: function (newAdmins) {
		if (newAdmins != null) {
			if (newAdmins.length > 0) {
				this.setDirty(true);
				if (this.pendingAdminChanges["add"] == null) {
					this.pendingAdminChanges["add"] = new Array();
				}
			}
			for (var i=0; i < newAdmins.length; i++) {
				var admin = newAdmins[i];
				admin.itemId = admin.itemId;
				admin.processRoles = [{id: "default"}];
				admin.savedProcessRoles = [{id: "default"}];
				this.allAdmins.push(admin);
				this.pendingAdminChanges.add.push(admin.itemId);
				if (this.pendingAdminChanges.remove != null) {
					for (var j=0; j < this.pendingAdminChanges.remove.length; j++) {
						if (this.pendingAdminChanges.remove[j] == admin.itemId) {
							this.pendingAdminChanges.remove.splice(j,1);
							break;
						}
					}
				}
			}
		}
		this.populateAdmins(this.allAdmins);
	},
	
	populateDevelopmentLine: function (data) {
		dojox.data.dom.removeChildren(this.developmentLineDiv);
		if (data != null && data.label != null) {
			this.developmentLineDiv.appendChild(document.createTextNode(data.label));
		}
		else {
			this.developmentLineDiv.appendChild(document.createTextNode("Development line data not found"));
		}
	},
	
	populateDevelopmentLineSelector: function () {
		dojox.data.dom.removeChildren(this.developmentLineDiv);
		if (this.parentProcessArea != null) {
			if (this.parentProcessArea.type == "projectArea") {
				var srh = new ServiceResponseHandler(this, "developmentLineSelectorCallback", "handleProjectDevLineQueryError");
				ProcessClient.getProjectDevelopmentLines({uuid: this.parentProcessArea.itemId}, srh);
			}
			else if (this.parentProcessArea.type == "teamArea") {
				var srh = new ServiceResponseHandler(this, "developmentLineDisplayCallback", "handleTeamDevLineQueryError");
				ProcessClient.getTeamAreaByUUID({uuid: this.parentProcessArea.itemId}, srh);
			}
		}
		else {
			this.developmentLineDiv.appendChild(document.createTextNode("Please choose a process hierarchy first"));
		}
	},
	
	handleProjectDevLineQueryError: function (err) {
		var msg = document.createElement("div");
		msg.appendChild(document.createTextNode("Error fetching development lines for parent project area.  Please check your server log for more details."));
		if (err.message != null && err.message != "") {
			msg.appendChild(document.createElement("br"));
			msg.appendChild(document.createElement("br"));
			msg.appendChild(document.createTextNode(err.message));
		}
		this.displayLongErrorMessage(msg);
	},
	
	handleTeamDevLineQueryError: function (err) {
		var msg = document.createElement("div");
		msg.appendChild(document.createTextNode("Error fetching development line for parent team area.  Please check your server log for more details."));
		if (err.message != null && err.message != "") {
			msg.appendChild(document.createElement("br"));
			msg.appendChild(document.createElement("br"));
			msg.appendChild(document.createTextNode(err.message));
		}
		this.displayLongErrorMessage(msg);
	},
	
	developmentLineSelectorCallback: function (data) {
		dojox.data.dom.removeChildren(this.developmentLineDiv);
		if (data != null && data.length > 0) {
			var select = document.createElement("select");
			this.developmentLineSelector = select;
			for (var i=0; i < data.length; i++) {
				var option = new Option(data[i].label, data[i].itemId);
				if (data[i].summary != null && data[i].summary != "") {
					option.description = data[i].summary;
				}
				else if (data[i].description != null && data[i].description != "") {
					option.description = data[i].description;
				}
				else {
					option.description = "No description provided for this development line";
				}
				if (dojo.isIE) {
					select.options[select.options.length] = option;
				}
				else {
					select.appendChild(option);
				}
			}
			this.developmentLineDiv.appendChild(select);
		}
		else {
			this.developmentLineDiv.appendChild(document.createTextNode("There are no development lines" +
					" associated with the selected project area."));
		}
	},
	
	developmentLineDisplayCallback: function (data) {
		this.populateDevelopmentLine(data.developmentLine);
	},
	
	populateProcessHierarchy: function (project) {
		dojox.data.dom.removeChildren(this.processHierarchy);
		this.showBreadcrumbs(this.isArchived, this.getAncestorsAsArray(project));
		var div = document.createElement("div");
		var link = document.createElement("a");
		link.href = "#action=" + PROJECT_AREA_EDITOR + "&itemId=" + project.itemId;
		link.tabIndex = 3;
		link.appendChild(document.createTextNode(project.name));
		div.appendChild(link);
		div.className = "hierarchyElement";
		this.processHierarchy.appendChild(div);
		var isNewParent = false;
		if (this.isNew && project.itemId === this.parentItemId) {
			this.parentProcessArea = {
				name: project.name,
				itemId: project.itemId,
				type: "projectArea"
			};
			isNewParent = true;
		}
		if (project.children != null && project.children.length > 0) {
			this.populateRemainingHierarchy(project.children, 1, isNewParent);
		}
		else {
			if (isNewParent) {
				this.populateRemainingHierarchy([], 1, true);
			}
		}
		this.populateDevelopmentLineSelector();
	},
	
	populateRemainingHierarchy: function (teams, indent, includeNewTeam) {
		teams.sort(function (a,b) {
			return a.name > b.name ? 1 : a.name < b.name ? -1 : 0;
		});
		dojo.forEach(teams, dojo.hitch(this, function (team) {
			var div = document.createElement("div");
			var link = document.createElement("a");
			link.href = "#action=" + TEAM_AREA_EDITOR + "&itemId=" + team.itemId;
			link.tabIndex = 3;
			link.appendChild(document.createTextNode(team.name));
			div.appendChild(link);
			div.className = "hierarchyElement";
			if (!this.isNew && this.itemId === team.itemId) {
				div.className += " selected";
				this.hierarchyNameDisplay = div;
			}
			div.style.paddingLeft = (15 * indent) + "px";
			this.processHierarchy.appendChild(div);
			var isNewParent = false;
			if (this.isNew && team.itemId === this.parentItemId) {
				this.parentProcessArea = {
					name: team.name,
					itemId: team.itemId,
					type: "teamArea"
				};
				isNewParent = true;
			}
			if (team.children != null && team.children.length > 0) {
				this.populateRemainingHierarchy(team.children, indent + 1, isNewParent);
			}
			else {
				if (isNewParent) {
					this.populateRemainingHierarchy([], indent + 1, true);
				}
			}
		}));
		if (includeNewTeam) {
			var div = document.createElement("div");
			this.hierarchyNameDisplay = div;
			div.className = "hierarchyElement selected";
			div.style.paddingLeft = (15 * indent) + "px";
			div.appendChild(document.createTextNode("(New Team Area)"));
			this.processHierarchy.appendChild(div);
		}
	},
	
	// returns an array representing the process hierarchy directly above this team area
	getAncestorsAsArray: function (processArea) {
		var foundParentOfNewTeam = this.isNew && this.parentItemId === processArea.itemId;
		var foundThisTeam = !this.isNew && this.itemId === processArea.itemId;
		var hasChildren = processArea.children && processArea.children.length > 0;
		
		// Return an array containing this process area so it can bubble up recursively
		// The recursive callers will insert their process area at the front of this array so
		// we get a complete ancestry array in the end 
		if (foundParentOfNewTeam || foundThisTeam) {
			return [ processArea ];
		}
		else if (hasChildren) {
			for (var i=0; i < processArea.children.length; i++) {
				var curLine = this.getAncestorsAsArray(processArea.children[i]);
				// If the current child team exists along the ancestry path, we know this element
				// also belongs along the ancestry path.  Add the current processArea to the front
				// of the ancestry path and bubble up to the next recursive caller.
				if (curLine) {
					curLine.unshift(processArea);
					return curLine;
				}
			}
		}
		else {
			// We didn't find what we were looking for down this branch.
			// Return null so the recursive callers will ignore this path.
			return null;
		}
	},
	
	showBreadcrumbs: function (isArchived, hierarchy) {
		dojox.data.dom.removeChildren(this.breadcrumbs);
		var link = document.createElement("a");
		link.href = isArchived ? "#action=" + ARCHIVED_MANAGER : "#action=" + MANAGER;
		link.appendChild(document.createTextNode(isArchived ? "Archived Project Areas" : "Active Project Areas"));
		this.breadcrumbs.appendChild(link);
		var span = document.createElement("span");
		span.appendChild(document.createTextNode(">"));
		this.breadcrumbs.appendChild(span);
		for (var i=0; i < hierarchy.length; i++) {
			// Don't include the current element in the breadcrumb hierarchy
			if (!this.isNew && i === hierarchy.length - 1) {
				continue;
			}
			link = document.createElement("a");
			if (i === 0) {
				link.href = "#action=" + PROJECT_AREA_EDITOR + "&itemId=" + hierarchy[i].itemId;
			}
			else {
				link.href = "#action=" + TEAM_AREA_EDITOR + "&itemId=" + hierarchy[i].itemId;
			}
			link.appendChild(document.createTextNode(hierarchy[i].name));
			this.breadcrumbs.appendChild(link);
			span = document.createElement("span");
			span.appendChild(document.createTextNode(">"));
			this.breadcrumbs.appendChild(span);
		}
	},
	
	handleChange: function (e) {
		if (e.target.value != e.target.savedValue) {
			e.target.changePending = true;
			this.setDirty(true);
		}
		else {
			e.target.changePending = false;
			if (!this.changesPending()) {
				this.setDirty(false);
			}
		}
	},
	
	setDirty: function (isDirty) {
		this.saveButton.disabled = !isDirty;
		if (isDirty) {
			this.modifiedFlag.style.display = "inline";
		}
		else {
			this.modifiedFlag.style.display = "none";
		}
	},
	
	changesPending: function () {
		for (var i=0; i < this.inputFields.length; i++) {
			if (this.inputFields[i].changePending) {
				return true;
			}
		}
		for (var i in this.pendingMemberChanges) {
			return true;
		}
		for (var i in this.pendingAdminChanges) {
			return true;
		}
		return false;
	},
	
	handleSave: function (e) {
		dojo.stopEvent(e);
		this.saveButton.disabled = true;
		var saveObj = {
			itemId: this.itemId
		};
		for (var i=0; i < this.inputFields.length; i++) {
			if (this.inputFields[i].changePending) {
				saveObj[this.inputFields[i].name] = this.inputFields[i].value;
			}
		}
		saveObj["jsonMembers"] = dojo.toJson(this.pendingMemberChanges);
		saveObj["jsonAdmins"] = dojo.toJson(this.pendingAdminChanges);
		if (this.isNew && this.parentProcessArea != null) {
			saveObj["jsonParentProcessArea"] = dojo.toJson(this.parentProcessArea);
		}
		if (this.developmentLineSelector != null && this.developmentLineSelector.value != null) {
			saveObj["developmentLineUuid"] = this.developmentLineSelector.value;
		}
		var msg = this.validateRequired(saveObj);
		if (msg != null && msg != "") {
			this.displayErrorMessage(msg);
		}
		else {
			var srh = new ServiceResponseHandler(this, "handleSaveResponse", "handleSaveError");
			ProcessClient.postTeamArea(saveObj, srh);
			this.displayPendingMessage("Save in progress...");
		}
	},
	
	validateRequired: function (saveData) {
		if (saveData.name == "" || (this.isNew && saveData.name == null)) {
			this.togglePrimaryField("Edit");
			return "The team area must have a name";
		}
		if (this.isNew && (saveData.jsonParentProcessArea == null || saveData.jsonParentProcessArea == "")) {
			return "You must select a process hierarchy";
		}
	},
	
	handleSaveResponse: function (response) {
		if (this.isNew && response) {
			dojo.cookie(MSG_COOKIE_NAME, "Team area created successfully.");
			PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
			this.navigateAwayHandle = null;
			PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);
			this.onBeforeUnloadHandle = null;
			document.location.href = "#action=" + TEAM_AREA_EDITOR + "&itemId=" + response;
		}
		else {
			this.refresh();
			this.displaySuccessMessage("Team area saved successfully.", true);
		}
	},
	
	handleSaveError: function (err) {
		this.saveButton.disabled = false;
		var msg = document.createElement("div");
		msg.appendChild(document.createTextNode("Saving team area failed.  Please see your server log for more detail."));
		if (err.message != null && err.message != "") {
			msg.appendChild(document.createElement("br"));
			msg.appendChild(document.createElement("br"));
			msg.appendChild(document.createTextNode(err.message));
		}
		this.displayLongErrorMessage(msg);
	},
	
	ignoreError: function (err) {},
	
	onRefresh: function(e){
		dojo.stopEvent(e);
		if (this.changesPending() && !confirm("Refreshing will discard any changes you have made.  Do you wish to continue?")) {
			return;
		}
		this.refresh();
	},
	
	displaySuccessMessage: function (msg, showManagerLink) {
		this.displayMessage(msg,"successMsg");
		if (showManagerLink) {
			this.messageBox.appendChild(document.createTextNode(" "));
			var link = document.createElement("a");
			link.href = "#action=" + MANAGER;
			link.appendChild(document.createTextNode("Return to Manage Projects & Teams"));
			this.messageBox.appendChild(link);
		}
	},
	
	displayPendingMessage: function (msg) {
		this.displayMessage(msg, "pendingMsg");
	},
	
	displayErrorMessage: function (msg) {
		this.displayMessage(msg, "errorMsg");
	},
	
	displayMessage: function (msg, cssClass) {
		dojox.data.dom.removeChildren(this.messageBox);
		dojox.data.dom.removeChildren(this.longMessageBox);
		this.longMessageBox.className = "";
		this.messageBox.className = "message shortMsg " + cssClass;
		this.messageBox.appendChild(document.createTextNode(msg));
	},
	
	displayLongSuccessMessage: function (msg) {
		if (dojo.isString(msg)) {
			msg = document.createTextNode(msg);
		}
		this.displayLongMessage(msg,"successMsg");
	},
	
	displayLongErrorMessage: function (msg) {
		if (dojo.isString(msg)) {
			msg = document.createTextNode(msg);
		}
		this.displayLongMessage(msg, "errorMsg");
	},
	
	displayLongMessage: function (msg, cssClass) {
		dojox.data.dom.removeChildren(this.messageBox);
		this.messageBox.className = "";
		dojox.data.dom.removeChildren(this.longMessageBox);
		this.longMessageBox.className = "message longMsg " + cssClass;
		this.longMessageBox.appendChild(msg);
	},
	
	destroy: function () {
		dojo.forEach(this.connectHandles, function (handle) {
			dojo.disconnect(handle);
		});
		this.inherited(arguments);
	}
});

})();